<?php

namespace Drupal\pki_ra\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\pki_ra\Misc\IdqaScore;

/**
 * PKI Registration Authority settings form for ID quality.
 */
class PKIRAIdQualitySettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'pki_ra_settings_id_quality_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['pki_ra.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('pki_ra.settings');

    $form['enrolment_score'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Enrolment score'),
      '#description' => $this->t('Initial ID Quality Assurance (IDQA) score assigned to the user upon enrolment, reported to the certification authority (CA) if the CA supports it. Optional. Example: <em>12345678</em>'),
      '#default_value' => $config->get('idqa.enrolment_score') ?: '',
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $score = trim($form_state->getValue(['enrolment_score']));
    if (!IdqaScore::isValid($score)) {
      $form_state->setErrorByName('enrolment_score', $this->t('Must be %digits digits exactly.', [
        '%digits' => IdqaScore::NUMBER_OF_DIGITS,
      ]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('pki_ra.settings')
      ->set('idqa.enrolment_score', $form_state->getValue('enrolment_score'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
